@@ -13,7 +13,6 @@
 DESCRIPTION = """
 """
 
-# import ipdb
 import cPickle as pickle
 import logging
 
@@ -23,6 +22,8 @@ from util import kmeans, pq_kmeans_assign, pq_knn
 from distance import distFunc
 from storage import PQ_DIC
 
+import _cext as cext
+
 
 class Indexer(object):
     class IdxData(object):
@@ -98,7 +99,7 @@ class PQIndexer(Indexer):
         nsubqbits = pardic.get('nsubqbits', 8)
         # the number of items in one block
 
-        # blksize = pardic.get('blksize', 16384)
+        blksize = pardic.get('blksize', 16384)
 
         # vector dimension
         dim = vals.shape[1]
@@ -120,6 +121,7 @@ class PQIndexer(Indexer):
         idxdat['nsubq'] = nsubq
         idxdat['ksub'] = ksub
         idxdat['dsub'] = dsub
+        idxdat['blksize'] = blksize
         idxdat['centroids'] = [None for q in range(nsubq)]
 
         logging.info("Building codebooks in subspaces - BEGIN")
@@ -141,29 +143,24 @@ class PQIndexer(Indexer):
         nsubq = self.idxdat['nsubq']
         centroids = self.idxdat['centroids']
 
-        codes = np.zeros((num_vals, nsubq), np.uint8)
-        for q in range(nsubq):
-            vsub = vals[:, q*dsub:(q+1)*dsub]
-            codes[:, q] = pq_kmeans_assign(centroids[q], vsub)
-        self.storage.add(codes, keys)
+        blksize = self.idxdat.get('blksize', 16384)
+        start_id = 0
+        for start_id in range(0, num_vals, blksize):
+            cur_num = min(blksize, num_vals - start_id)
+            print "%8d/%d: %d" % (start_id, num_vals, cur_num)
 
-#         start_id = 0
-#         while start_id < num_vals:
-#             if self.num_emptys is 0:
-#                 self.blocks.append(np.zeros((blksize, nsubq), np.uint8))
-#                 self.num_emptys = blksize
-#             cur_num = min(self.num_emptys, num_vals-start_id)
-#             cur_start = blksize - self.num_emptys
-#
-#             for q in range(nsubq):
-#                 vsub = vals[start_id:start_id+cur_num, q*dsub:(q+1)*dsub]
-#                 # ipdb.set_trace()
-#                 self.blocks[-1][cur_start:cur_start+cur_num, q] = \
-#                     pq_kmeans_assign(centroids[q], vsub)
-#
-#             self.num_emptys -= cur_num
-#             start_id += cur_num
-        # ipdb.set_trace()
+            codes = np.zeros((cur_num, nsubq), np.uint8)
+            for q in range(nsubq):
+                vsub = vals[start_id:start_id+cur_num, q*dsub:(q+1)*dsub]
+                codes[:, q] = pq_kmeans_assign(centroids[q], vsub)
+            # self.storage.add(codes, keys[start_id:start_id+cur_num])
+            self.storage.add(codes, None)
+
+        # codes = np.zeros((num_vals, nsubq), np.uint8)
+        # for q in range(nsubq):
+        #     vsub = vals[:, q*dsub:(q+1)*dsub]
+        #     codes[:, q] = pq_kmeans_assign(centroids[q], vsub)
+        # self.storage.add(codes, keys)
 
     def remove(self, keys):
         raise Exception(self.INST_ERR)
@@ -202,7 +199,6 @@ class PQIndexer(Indexer):
             D: nsubq x ksub
         """
 
-        nsubq = D.shape[0]
         num_base_items = self.storage.get_num_items()
 
         dis = np.zeros(num_base_items)
@@ -210,9 +206,13 @@ class PQIndexer(Indexer):
 
         for keys, blk in self.storage:
             cur_num = blk.shape[0]
-            dis[start_id:start_id+cur_num] = \
-                [sum([D[j, blk[i, j]] for j in range(nsubq)])
-                 for i in range(cur_num)]
-            start_id += cur_num
+            # dis[start_id:start_id+cur_num] = self.sumidxtab_core(D, blk)
+            dis[start_id:start_id+cur_num] = cext.sumidxtab_core(D, blk)
 
         return dis
+
+    @classmethod
+    def sumidxtab_core(cls, D, blk):
+        # return 0
+        return [sum([D[j, blk[i, j]] for j in range(D.shape[0])])
+                for i in range(blk.shape[0])]
